@@ -68,6 +68,7 @@ class PetscConfig:
         confstr += 'PETSC_ARCH = %s\n' % petsc_arch
         confstr += open(petscvars).read()
         confstr += open(petscconf).read()
+        confstr += 'PACKAGES_INCLUDES = ${MPI_INCLUDE} ${X11_INCLUDE} ${BLASLAPACK_INCLUDE}'
         confstr += 'PACKAGES_LIBS = ${MPI_LIB} ${X11_LIB} ${BLASLAPACK_LIB}\n'
         confdct = cfgutils.makefile(StringIO(confstr))
         return confdct
@@ -106,25 +107,26 @@ class PetscConfig:
         # includes and libraries
         petsc_inc = cfgutils.flaglist(self['PETSC_INCLUDE'])
         petsc_lib = cfgutils.flaglist(self['PETSC_LIB'])
-        self._configure_ext(extension, petsc_inc)
+        self._configure_ext(extension, petsc_inc, preppend=True)
         self._configure_ext(extension, petsc_lib)
         # extra configuration
         cflags = self['PCC_FLAGS'].split()
-        try:
+        if '-Wwrite-strings' in cflags:
             cflags.remove('-Wwrite-strings')
-        except ValueError:
-            pass
         extension.extra_compile_args.extend(cflags)
         lflags = []
         extension.extra_link_args.extend(lflags)
         
-    def _configure_ext(self, ext, dct):
+    def _configure_ext(self, ext, dct, preppend=False):
         extdict = ext.__dict__
         for key, values in dct.items():
             if key in extdict:
                 for value in values:
                     if value not in extdict[key]:
-                        extdict[key].append(value)
+                        if preppend:
+                            extdict[key].insert(0, value)
+                        else:
+                            extdict[key].append(value)
 
     def _configure_compiler(self, compiler):
         pass
@@ -306,7 +308,7 @@ class build_src(_build_src):
                   '-DPETSC_ARCH=%s' % self.petsc_arch[0]]
         self.swigflags.extend(macros)
         self.swigflags.append('-O')
-        self.swigflags.append('-fcompact')
+        #self.swigflags.append('-fcompact')
         #self.swigflags.append('-noproxy')
 
     def swig_sources(self, sources, extension):
