@@ -29,8 +29,15 @@ __docformat__ = 'reST'
 # --------------------------------------------------------------------
 
 __all__ = ['KSP',
+           'KSPCG',
+           'KSPGMRES',
+           
            'PC',
-           'PCShell']
+           'PCNone',
+           'PCLU',
+           'PCILU',
+           'PCShell',
+           ]
 
 # --------------------------------------------------------------------
 
@@ -48,7 +55,7 @@ class KSP(Object):
     Abstract PETSc object that manages all Krylov methods.
     """
     
-    class Type:
+    class Type(object):
         """
         KSP Types
         """
@@ -74,6 +81,25 @@ class KSP(Object):
         SYMMLQ     = _petsc.KSPSYMMLQ
         LCD        = _petsc.KSPLCD
 
+        def __init__(self, ksp_type):
+            self._ksp_type = ksp_type
+        def __call__(self, name, bases, dct):
+            klass = type(KSP)(name, bases, dct)
+            setattr(klass, 'TYPE', self._ksp_type)
+            def __init__(self, ksp=None, *targs, **kwargs):
+                """See `KSP.create()` and KSP.setType()."""
+                super(KSP, self).__init__(*targs, **kwargs)
+                ksp_type = getattr(type(self), 'TYPE', None)
+                if isinstance(ksp, KSP):
+                    _petsc.KSP_init(self, ksp, ksp_type)
+                else:
+                    comm = ksp
+                    _petsc.KSPCreate(comm, self)
+                    if ksp_type is not None:
+                        _petsc.KSPSetType(self, ksp_type)
+            setattr(klass, '__init__', __init__)
+            return klass
+
     class NormType:
         """
         Norm that is passed in the Krylov convergence test routines.
@@ -487,6 +513,29 @@ class KSP(Object):
     convstr   = property(getConvergedReasonString)
 
 
+# --------------------------------------------------------------------
+
+class KSPPREONLY(KSP):
+    """
+    Implements a stub method that applies ONLY the preconditioner.
+    """
+    __metaclass__ = KSP.Type(KSP.Type.PREONLY)
+
+
+class KSPCG(KSP):
+    """
+    Implements the Conjugate Gradients method.
+    """
+    __metaclass__ = KSP.Type(KSP.Type.CG)
+
+
+class KSPGMRES(KSP):
+    """
+    Implements the Generalized Minimal Residual method.
+    """
+    __metaclass__ = KSP.Type(KSP.Type.GMRES)
+            
+
 # --------------------------------------------------------------------
 
 
@@ -496,7 +545,7 @@ class PC(Object):
     Abstract PETSc object that manages all preconditioners.
     """
 
-    class Type:
+    class Type(object):
         """
         PC Types
         """
@@ -529,6 +578,25 @@ class PC(Object):
 
         SCHUR      = _petsc.PCSCHUR
 
+        def __init__(self, pc_type):
+            self._pc_type = pc_type
+        def __call__(self, name, bases, dct):
+            klass = type(PC)(name, bases, dct)
+            setattr(klass, 'TYPE', self._pc_type)
+            def __init__(self, pc=None, *targs, **kwargs):
+                """See `PC.create()` and PC.setType()."""
+                super(PC, self).__init__(*targs, **kwargs)
+                pc_type = getattr(type(self), 'TYPE', None)
+                if isinstance(pc, PC):
+                    _petsc.PC_init(self, pc, pc_type)
+                else:
+                    comm = pc
+                    _petsc.PCCreate(comm, self)
+                    if pc_type is not None:
+                        _petsc.PCSetType(self, pc_type)
+            setattr(klass, '__init__', __init__)
+            return klass
+
     class Side:
         """
         PC sides
@@ -648,6 +716,17 @@ class PC(Object):
         """
         _petsc.PCApplyTranspose(self, x, y)
 
+    # XXX this should go to subtypes
+    
+    def getFactoredMatrix(self):
+        """
+        Gets the factored matrix from the preconditioner context.
+
+        ..note:: This routine is valid only for the LU, incomplete LU,
+                 Cholesky, and incomplete Cholesky methods.
+        """
+        return _petsc.PCGetFactoredMatrix(self)
+
     def getShellContext(self):
         """
         Get the user-provided context object associated with a shell
@@ -663,33 +742,62 @@ class PC(Object):
         _petsc.PCShellSetContext(self, context)
     
 
-class PCShell(PC):
+# --------------------------------------------------------------------
 
+
+class PCNone(PC):
+    """
+    This is used when you wish to employ a nonpreconditioned Krylov
+    method.
+    """
+    __metaclass__ = PC.Type(PC.Type.NONE)
+
+class PCLU(PC):
+    """
+    Use a direct solver, based on LU factorization, as a
+    preconditioner.
+    """
+    __metaclass__ = PC.Type(PC.Type.LU)
+
+class PCILU(PC):
+    """
+    Incomplete LU factorization preconditioner.
+    """
+    __metaclass__ = PC.Type(PC.Type.ILU)
+
+class PCCHOLESKY(PC):
+    """
+    Use a direct solver, based on Cholesky factorization, as a
+    preconditioner.
+    """
+    __metaclass__ = PC.Type(PC.Type.CHOLESKY)
+
+class PCICC(PC):
+    """
+    Incomplete Cholesky factorization preconditioner.
+    """
+    __metaclass__ = PC.Type(PC.Type.ICC)
+
+class PCShell(PC):
     """
     A PC type to be used to define your own preconditioning
     operator.
     """
-
-    def __init__(self, *targs, **kwargs):
-        """See `PC.createShell()`."""
-        super(PC, self).__init__(*targs, **kwargs)
-        self.createShell(*targs, **kwargs)
+    __metaclass__ = PC.Type(PC.Type.SHELL)
             
-    @staticmethod
-    def GetContext(pc):
+    def getContext(self):
         """
         Get the user-provided context object associated with a shell
         preconditioner.
         """
-        return _petsc.PCShellGetContext(pc)
+        return _petsc.PCShellGetContext(self)
 
-    @staticmethod
-    def SetContext(pc, context):
+    def setContext(self, context):
         """
         Set the user-provided context object associated with a shell
         preconditioner.
         """
-        _petsc.PCShellSetContext(pc, context)
+        _petsc.PCShellSetContext(self, context)
 
     def getName(self):
         """
@@ -705,4 +813,5 @@ class PCShell(PC):
         """
         _petsc.PCShellSetName(self, name)
 
+
 # --------------------------------------------------------------------
