@@ -27,7 +27,7 @@ __all__ = ['Vec',
            'VecSeq',
            'VecMPI',
            'VecShared',
-           'VecGhost',
+           'VecSieve',
 
            'Scatter',
            ]
@@ -43,14 +43,13 @@ from petsc4py.Object import Object
 # --------------------------------------------------------------------
 
 
-
 class Vec(Object):
 
     """
     Abstract PETSc vector object.
     """
 
-    class Type:
+    class Type(object):
         """
         Vector types
         """
@@ -58,6 +57,20 @@ class Vec(Object):
         MPI    = _petsc.VECMPI
         FETI   = _petsc.VECFETI
         SHARED = _petsc.VECSHARED
+        SIEVE  = _petsc.VECSIEVE
+        def __init__(self, vec_type):
+            self._vec_type = vec_type
+        def __call__(self, name, bases, dct):
+            klass = type(Vec)(name, bases, dct)
+            setattr(klass, 'TYPE', self._vec_type)
+            if '__init__' not in dct:
+                def __init__(self, vec=None, *targs, **kwargs):
+                    """See `Vec.create()` and Vec.setType()."""
+                    super(Vec, self).__init__(*targs, **kwargs)
+                    vec_type = getattr(type(self), 'TYPE', None)
+                    _petsc.Vec_init(self, vec, vec_type)
+                setattr(klass, '__init__', __init__)
+            return klass
 
     class Option:
         """
@@ -768,66 +781,43 @@ class Vec(Object):
     # Array Interface V3, Python side
     __array_interface__ = property(_petsc.Vec__array_interface__,
                                    doc='Array protocol: interface')
-    
-    # Array Interface V2, Python side
-    __array_shape__    = property(_petsc.Vec__array_shape__,
-                                  doc='Array protocol: shape')
-    __array_data__     = property(_petsc.Vec__array_data__,
-                                  doc='Array protocol: data')
-    __array_typestr__  = property(_petsc.Vec__array_typestr__,
-                                  doc='Array protocol: typestr')
-
-class VecSeq(Vec):
-
-    """
-    Sequential vector implementation.
-    """
-
-    def __init__(self, *targs, **kwargs):
-        """See `Vec.createSeq()`"""
-        super(VecSeq, self).__init__(*targs, **kwargs)
-        self.createSeq(*targs, **kwargs)
-        
-
-class VecMPI(Vec):
-
-    """
-    Parallel vector implementation.
-    """
-
-    def __init__(self, *targs, **kwargs):
-        """See `Vec.createMPI()`"""
-        super(VecMPI, self).__init__(*targs, **kwargs)
-        self.createMPI(*targs, **kwargs)
-
-
-class VecShared(Vec):
-
-    """
-    Parallel vector implementation that uses shared memory.
-    """
-
-    def __init__(self, *targs, **kwargs):
-        """See `Vec.createShared()`"""
-        super(VecShared, self).__init__(*targs, **kwargs)
-        self.createShared(*targs, **kwargs)
-
-
-class VecGhost(VecMPI):
-
-    """
-    Parallel vector with ghost padding on each processor.
-    """
-
-    def __init__(self, *targs, **kwargs):
-        """See `Vec.createGhost()`"""
-        super(VecMPI, self).__init__(*targs, **kwargs)
-        self.createGhost(*targs, **kwargs)
-
-
 
 # --------------------------------------------------------------------
 
+class VecSeq(Vec):
+    """
+    Sequential vector implementation.
+    """
+    __metaclass__ = Vec.Type(Vec.Type.SEQ)
+    def __init__(self, *targs, **kwargs):
+        super(VecSeq, self).__init__(*targs, **kwargs)
+        self.createSeq(*targs, **kwargs)
+        
+class VecMPI(Vec):
+    """
+    Parallel vector implementation.
+    """
+    __metaclass__ = Vec.Type(Vec.Type.MPI)
+    def __init__(self, *targs, **kwargs):
+        super(VecMPI, self).__init__(*targs, **kwargs)
+        self.createMPI(*targs, **kwargs)
+
+class VecShared(Vec):
+    """
+    Parallel vector implementation that uses shared memory.
+    """
+    __metaclass__ = Vec.Type(Vec.Type.SHARED)
+    def __init__(self, *targs, **kwargs):
+        super(VecShared, self).__init__(*targs, **kwargs)
+        self.createShared(*targs, **kwargs)
+
+class VecSieve(Vec):
+    """
+    The parallel vector implementation based upon Sieve fields
+    """
+    __metaclass__ = Vec.Type(Vec.Type.SIEVE)
+
+# --------------------------------------------------------------------
 
 
 class Scatter(Object):
